@@ -7,15 +7,16 @@
  * Contract with IBM Corp.
  ****************************************************************************** */
 /* Copyright (c) 2020 Red Hat, Inc. */
+/* Copyright Contributors to the Open Cluster Management project */
 
 import express from 'express';
-import { ApolloServer } from 'apollo-server-express';
+import { ApolloServer, AuthenticationError } from 'apollo-server-express';
 import { isInstance as isApolloErrorInstance, formatError as formatApolloError } from 'apollo-errors';
 import morganBody from 'morgan-body';
 import helmet from 'helmet';
 import compression from 'compression';
 import cookieParser from 'cookie-parser';
-import inspect from 'security-middleware';
+import inspect from '@open-cluster-management/security-middleware';
 import noCache from 'nocache';
 import authMiddleware from './lib/auth-middleware';
 import logger from './lib/logger';
@@ -23,6 +24,7 @@ import KubeConnector from './connectors/kube';
 import ClusterModel from './models/cluster';
 import GenericModel from './models/generic';
 import ComplianceModel from './models/compliance';
+import AnsibleModel from './models/ansible';
 import schema from './schema';
 import config from '../../config';
 
@@ -47,10 +49,22 @@ const apolloServer = new ApolloServer({
     endpoint: GRAPHQL_PATH,
   },
   context: ({ req }) => {
+    if (req.user.namespaces.code === 401) {
+      logger.error('Found 401 unauthenticated returned for user namespaces');
+      throw new AuthenticationError('Unauthenticated');
+    }
     const namespaces = req.user.namespaces.items.map((ns) => ns.metadata.name);
+    const clusterNamespaces = req.user.namespaces.items
+      .filter((ns) => ns.metadata.labels && ns.metadata.labels['cluster.open-cluster-management.io/managedCluster'] === ns.metadata.name)
+      .map((ns) => ns.metadata.name);
+    const userNamespaces = req.user.namespaces.items
+      .filter((ns) => !ns.metadata.labels || !ns.metadata.labels['cluster.open-cluster-management.io/managedCluster'])
+      .map((ns) => ns.metadata.name);
     const kubeConnector = new KubeConnector({
       token: req.kubeToken,
       namespaces,
+      clusterNamespaces,
+      userNamespaces,
     });
     if (isTest) {
       kubeConnector.kubeApiEndpoint = 'http://0.0.0.0/kubernetes';
@@ -61,6 +75,7 @@ const apolloServer = new ApolloServer({
       clusterModel: new ClusterModel({ kubeConnector }),
       genericModel: new GenericModel({ kubeConnector }),
       complianceModel: new ComplianceModel({ kubeConnector }),
+      ansibleModel: new AnsibleModel({ kubeConnector }),
     };
   },
 });
